'use strict';
import React, { Component } from 'react';
import {
    AppRegistry,
    StyleSheet,
    WebView,
    View, AsyncStorage
} from 'react-native';
import { Actions } from 'react-native-router-flux';
import * as Titles from '../titles';
import * as Validation from '../validation';
import * as UIA from '../actions/uiActions';
import * as Common from '../common';
const UPDATE_SUB_OBJECT = 'UPDATE_SUB_OBJECT';
const NEW_SUB_OBJECT = 'NEW_SUB_OBJECT';

import {
    Container,
    Header,
    Footer,
    FooterTab,
    Body,
    Spinner,
    Button,
    Content,
    List,
    ListItem,
    Text
} from 'native-base';
// Change these to reflect
export default class RedScreenComponent extends Component {
    constructor(props) {
        super(props);
        this.state = {
            objectValue: null,
            validationState: {}
        };
    }
    componentDidMount() {
        var me = this;
        me.props.getUIDefinition(me.props.objectType);
        if (me.props.id) {
            var id = me.props.id;
            me.props.getRedObject(me.props.objectType, id, obj => {
                if (obj) {
                    me.setState({ objectValue: obj });
                }
            });
        }
        else if (me.props.mode === UIA.MODE.UPDATE && me.props.item) {
            me.setState({ objectValue: me.props.item });
        }
        else {
            me.props.getDefaultRedObject(me.props.objectType, obj => {
                if (obj) {
                    me.setState({ objectValue: obj })
                }
            })
        }
    }
    isValid() {
        var me = this;
        var {state} = me.props;
        var ui = UIA.GetC(state, UIA.UI_DEFINIONS, me.props.objectType);
        var validationResult = Validation.validateUI(ui, me.state.objectValue);
        if (validationResult && !validationResult.result) {
            me.setState({ validationState: validationResult.validationState });
            return false;;
        }
        return true;
    }
    newObject(newObject, callback) {
        var me = this;
        if (me.newObjectAction) {
            me.newObjectAction({
                objectType: newObject,
                mode: UIA.MODE.CREATE,
                callback,
                action: NEW_SUB_OBJECT
            });
        }
    }
    onCreatedObject(newObject, item) {
        var me = this;
        if (me.onCreatedObjectAction) {
            me.onCreatedObjectAction({
                objectType: newObject,
                item
            });
        }
    }
    updateObject(newObject, item, callback) {
        var me = this;
        if (me.updateObjectAction) {
            me.updateObjectAction({
                objectType: newObject,
                item,
                mode: UIA.MODE.UPDATE,
                callback,
                action: UPDATE_SUB_OBJECT
            });
        }
    }
    onUpdatedObject(newObject, item, index) {
        var me = this;
        if (me.onUpdatedObjectAction) {
            me.onUpdatedObjectAction({
                objectType: newObject,
                item,
                index
            });
        }
    }
    onDeletedObject(objType, item, index) {
        var me = this;
        if (me.onDeletedObjectAction) {
            me.onDeletedObjectAction({
                objectType: objType,
                item,
                index
            });
        }
    }
    getFooter() {
        var me = this;
        var result = null;
        var { mode, state } = me.props;
        var onCreate = me.props.onCreate || me.onCreate;
        var onUpdate = me.props.onUpdate || me.onUpdate;
        var afterUpdate = value => {
            if (onUpdate) {
                onUpdate(value);
            }
            if (!me.props.skipPopOnUpdate) {
                Actions.pop();
            }
        };
        switch (mode) {
            case UIA.MODE.CREATE:
                result = (
                    <Footer>
                        <FooterTab>
                            <Button onPress={() => {
                                Actions.pop();
                                if (me.props.callback) {
                                    me.props.callback(null);
                                }
                            }}>
                                <Text>{Titles.Back}</Text>
                            </Button>
                            <Button onPress={() => {

                                var ui = UIA.GetC(state, UIA.UI_DEFINIONS, me.props.objectType);
                                var validationResult = Validation.validateUI(ui, me.state.objectValue);
                                if (validationResult && !validationResult.result) {
                                    me.setState({ validationState: validationResult.validationState });
                                    return;
                                }
                                if (me.props.callback) {
                                    me.props.callback(Object.assign({}, me.state.objectValue, {
                                        machineOwned: UIA.Get(state, UIA.UNIQUE_MACHINE_SIG)
                                    }));
                                    if (!me.props.skipPopOnCreate && !me.skipPopOnCreate) {
                                        Actions.pop();
                                    }
                                }
                                else {
                                    me.props.createRedObject(me.props.objectType, Object.assign({}, me.state.objectValue, {
                                        machineOwned: UIA.Get(state, UIA.UNIQUE_MACHINE_SIG)
                                    }), value => {
                                        if (onCreate) {
                                            onCreate(value);
                                        }
                                        if (!me.props.skipPopOnCreate && !me.skipPopOnCreate) {
                                            Actions.pop();
                                        }
                                    }, error => {
                                        if (me.props.onError) {
                                            me.props.onError(error);
                                        }
                                    });
                                }
                            }}>
                                <Text>{Titles.Create}</Text>
                            </Button>
                        </FooterTab>
                    </Footer>
                );
                break;
            case UIA.MODE.UPDATE:
                result = (
                    <Footer>
                        <FooterTab>
                            <Button onPress={Actions.pop}>
                                <Text>{Titles.Back}</Text>
                            </Button>
                            <Button onPress={() => {
                                var ui = UIA.GetC(state, UIA.UI_DEFINIONS, me.props.objectType);
                                var validationResult = Validation.validateUI(ui, me.state.objectValue);
                                if (validationResult && !validationResult.result) {
                                    me.setState({ validationState: validationResult.validationState });
                                    return;
                                }

                                if (me.props.callback) {
                                    me.props.callback(Object.assign({}, me.state.objectValue, {
                                        machineOwned: UIA.Get(state, UIA.UNIQUE_MACHINE_SIG)
                                    }));
                                    Actions.pop();
                                }
                                else {
                                    me.props.updateRedObject(me.props.objectType, me.props.id, me.state.objectValue, afterUpdate);
                                }
                            }}>
                                <Text>{Titles.Save}</Text>
                            </Button>
                        </FooterTab>
                    </Footer>
                );
                break;
            case UIA.MODE.READ:
                result = (
                    <Footer>
                        <FooterTab>
                            {me.state.edit ? null : (<Button onPress={Actions.pop}>
                                <Text>{Titles.Back}</Text>
                            </Button>)}
                            {!me.state.edit ? null : (<Button onPress={() => {

                                var ui = UIA.GetC(state, UIA.UI_DEFINIONS, me.props.objectType);
                                var validationResult = Validation.validateUI(ui, me.state.objectValue);
                                if (validationResult && !validationResult.result) {
                                    me.setState({ validationState: validationResult.validationState });
                                    return;
                                }

                                me.props.updateRedObject(me.props.objectType, me.props.id, me.state.objectValue, afterUpdate);
                            }}>
                                <Text>{Titles.Save}</Text>
                            </Button>)}
                            <Button onPress={() => {
                                me.setState({ edit: !me.state.edit })
                            }}>
                                <Text>{!me.state.edit ? Titles.Edit : Titles.Cancel}</Text>
                            </Button>
                        </FooterTab>
                    </Footer>
                );
                break;
            case UIA.MODE.DELETE:
                result = (
                    <Footer>
                        <FooterTab>
                            <Button onPress={Actions.pop}>
                                <Text>{Titles.Back}</Text>
                            </Button>
                            <Button onPress={() => {
                                me.props.deleteRedObject(me.props.objectType, me.props.id);
                            }}>
                                <Text>{Titles.Delete}</Text>
                            </Button>
                        </FooterTab>
                    </Footer>
                );
                break;
        }

        return result;
    }
    getItemOptions() {
        return {};
    }
    getTitle(item) {
    }
    getSubTitle(item) {

    }
    getRightHeader() { return null; }
    getBody(item) {

    }
    getThumbnail(item) {
        return null;
    }
    getValidation() {
        var me = this;
        return Object.assign({
            get: () => {
                return me.state.validationState
            },
            set: (name, value, callback) => {
                var validationState = Object.assign({}, me.state.validationState, name ? { [name]: value } : value);
                me.setState({ validationState }, callback)
            },
            newObject: (objType, callback) => {
                me.newObject(objType, callback);
            },
            onCreatedObject: (objType, obj) => {
                me.onCreatedObject(objType, obj);
            },
            updateObject: (objType, item, callback) => {
                me.updateObject(objType, item, callback);
            },
            onUpdatedObject: (objType, item, index) => {
                me.onUpdatedObject(objType, item, index);
            },
            onDeletedObject: (objType, item, index) => {
                me.onDeletedObject(objType, item, index);
            },
            getCurrentMode: () => me.props.mode,
            isEditing: () => me.state.edit,
            getTitle: me.getTitle,
            getSubTitle: me.getSubTitle,
            getBody: me.getBody,
            getThumbnail: me.getThumbnail,
            carmakes: () => {
                var makes = UIA.Get(state, UIA.CARMAKES);
                if (!makes || !makes.length) {
                    me.props.getCarMakes();
                }
                return makes || [];
            },
            carmodels: (make) => {
                var models = UIA.GetC(state, UIA.CARMODELS, make);
                if (!models || !models.length) {
                    me.props.getCarModels(make);
                }
                return models || [];
            },
            vin: (val, callback) => {
                var _vin = UIA.GetC(state, UIA.VINS, val);
                if (!_vin) {
                    me.props.getVin(val, callback);
                }
                return _vin || null;
            }
        }, me.getItemOptions())
    }
    render() {
        var me = this;
        var { state } = me.props;
        var ui = UIA.GetC(state, UIA.UI_DEFINIONS, me.props.objectType);
        var footer = me.getFooter();
        return (
            <Container>
                <Header>
                    {Common.BackButton()}
                    {Common.HeaderTitle(me.props.title)}
                    {me.getRightHeader()}
                </Header>
                <Content>
                    {UIA.GetC(state, UIA.LOADING_RED_OBJECT, me.props.objectType) ? <Spinner /> : null}
                    {ui ? Common.CommonUi(ui, () => {
                        return me.state.objectValue;
                    }, (name, value, calback) => {
                        calback = calback || (() => { });
                        var objectValue = Object.assign({}, me.state.objectValue, name ? { [name]: value } : value);
                        me.setState({ objectValue }, () => calback(objectValue))
                        return objectValue;
                    }, me.props.mode === UIA.MODE.READ && !me.state.edit,
                        me.getValidation()
                    ) : null}
                </Content>
                {footer}
            </Container>
        )
    }
}

const styles = ({
});
